import os
import imp
import mistune
import re
from olm.helper import Map, merge_dictionaries
from olm.logger import get_logger
from jinja2 import Environment, FileSystemLoader, select_autoescape

logger = get_logger('olm.context')

def load_default_context(path):
    logger.info('Loading default context')
    BASE_FOLDER         = os.path.abspath(path)
    SOURCE_FOLDER       = os.path.abspath(os.path.join(BASE_FOLDER, 'src'))
    STATIC_FOLDER       = os.path.abspath(os.path.join(BASE_FOLDER, 'theme', 'static'))
    CSS_FOLDER          = os.path.abspath(os.path.join(BASE_FOLDER, 'theme', 'static', 'css'))
    JS_FOLDER           = os.path.abspath(os.path.join(BASE_FOLDER, 'theme', 'static', 'js'))
    TEMPLATES_FOLDER    = os.path.abspath(os.path.join(BASE_FOLDER, 'theme', 'templates'))
    PLUGINS_FOLDER      = os.path.abspath(os.path.join(BASE_FOLDER, 'plugins',))
    MD                  = mistune.Markdown()
    JINJA_ENV           = Environment(
                            loader=FileSystemLoader([TEMPLATES_FOLDER])
                            )
    ARTICLE_TYPES       = ['article']
    INDEX_TYPES         = ['index']
    PLUGINS             = []
    SITEURL             = ""
    OUTPUT_FOLDER       = os.path.abspath(os.path.join(BASE_FOLDER, 'dist'))
    OUTPUT_CSS_FOLDER   = os.path.abspath(os.path.join(OUTPUT_FOLDER, 'theme', 'css'))
    OUTPUT_JS_FOLDER    = os.path.abspath(os.path.join(OUTPUT_FOLDER, 'theme', 'js'))
    NO_SCAN             = []

    CONTEXT = Map({
        "BASE_FOLDER":          BASE_FOLDER,
        "SOURCE_FOLDER":        SOURCE_FOLDER,
        "OUTPUT_FOLDER":        OUTPUT_FOLDER,
        "STATIC_FOLDER":        STATIC_FOLDER,
        "CSS_FOLDER":           CSS_FOLDER,
        "JS_FOLDER":            JS_FOLDER,
        "MD":                   MD,
        "JINJA_ENV":            JINJA_ENV,
        "ARTICLE_TYPES":        ARTICLE_TYPES,
        "INDEX_TYPES":          INDEX_TYPES,
        "PLUGINS":              PLUGINS,
        "PLUGINS_FOLDER":       PLUGINS_FOLDER,
        "SITEURL":              SITEURL,
        "SUBSITES":             {},
        "OUTPUT_CSS_FOLDER":    OUTPUT_CSS_FOLDER,
        "OUTPUT_JS_FOLDER":     OUTPUT_JS_FOLDER,
        "NO_SCAN":              NO_SCAN
    })

    return CONTEXT

def load_context(CONTEXT, settings_file_path=None, settings=None):
    logger.info('Loading site context')

    # Load the file and extract the settings dict
    if (settings_file_path is not None):
        py_mod = imp.load_source('settings', settings_file_path)
        user_settings = getattr(py_mod, 'SETTINGS')
    else:
        user_settings = settings
    
    # Combine default and site settings
    CONTEXT = Map(merge_dictionaries(CONTEXT, user_settings))

    for key in CONTEXT:
        # Do variable substitutions in settings strings
        if isinstance(CONTEXT[key], str):
            value = CONTEXT[key]
            pattern = re.compile(r'{{\s*(\S*)\s*}}')
            for match in re.finditer(pattern, CONTEXT[key]):
                try:
                    value = value.replace(match.group(0), CONTEXT[match.group(1)])
                except KeyError as e:
                    logger.warn("Can't replace '%s' in settings file for key %s", match.group(1), key)
                    logger.warn(e)
            CONTEXT[key] = value

        # Reset the Jinja Env if we have a new templates folder
        if key == "TEMPLATES_FOLDER":
            CONTEXT["JINJA_ENV"] = Environment(
                loader=FileSystemLoader([CONTEXT[key]])
            )
    return CONTEXT



import logging, verboselogs, coloredlogs

def get_logger(name=''):
    logger = verboselogs.VerboseLogger(name)
    coloredlogs.install(level='INFO', logger=logger, fmt='%(asctime)s [%(name)s] %(message)s')
    return logger


import sys
import os
import time
import codecs
import sass
from jsmin import jsmin
from shutil import copyfile

from olm.context import load_context, load_default_context
from olm.plugins import Plugins
from olm.article import Article
from olm.index import Index
from olm.page import Page
from olm.constants import ArticleStatus
from olm.helper import Map
from olm.writer import Writer
from olm.signals import Signal, signals
from olm.logger import get_logger

logger = get_logger('olm.site')

class Site:
    def __init__(self, context):
        self.context = context
        self.subsites = []
    
    def build_site(self):
        # Source markdown files
        CONTEXT = self.context
        all_files = []
        articles = []
        draft_articles = []
        unlisted_articles = []
        pages = []
        subsites = set()

        CONTEXT['authors'] = {}
        CONTEXT['all_files'] = []
        CONTEXT['articles'] = []
        CONTEXT['pages'] = []

        signal_sender = Signal(signals.SITE_INITIALISED)
        signal_sender.send(context=CONTEXT)

        logger.info("Scanning source files")
        time_source_start = time.time()
        for dirname, dirs, files in os.walk(CONTEXT.SOURCE_FOLDER):
            for filename in files:
                filepath = os.path.join(dirname, filename)
                relpath = os.path.relpath(filepath, CONTEXT.SOURCE_FOLDER)
                firstfolder = relpath.split(os.sep)[0]
                basename, extension = os.path.splitext(filename)
                if firstfolder in CONTEXT.NO_SCAN:
                    continue
                if extension.lower() == ".md":
                    if firstfolder[0] == "_":
                        subsites.add(firstfolder)
                    elif firstfolder == "pages":
                        logger.debug("Found %s", filepath)
                        page = Page(CONTEXT, filepath)
                        pages.append(page)
                        all_files.append(page)
                    else:
                        logger.debug("Found %s", filepath)
                        article = Article(CONTEXT, filepath)
                        if article.type in CONTEXT.ARTICLE_TYPES + CONTEXT.INDEX_TYPES:
                            if article.status == ArticleStatus.ACTIVE:
                                articles.append(article)
                            elif article.status == ArticleStatus.UNLISTED:
                                unlisted_articles.append(article)
                            else:
                                draft_articles.append(article)
                        all_files.append(article)
        logger.info("Processed %d articles, %d unlisted articles, %d drafts, and %d pages in %.3f seconds", len(articles), len(unlisted_articles), len(draft_articles), len(pages), time.time() - time_source_start)            

        CONTEXT['all_files'].extend(all_files)
        CONTEXT['articles'].extend(sorted(articles, key=lambda k: (k.date), reverse=True))
        CONTEXT['pages'].extend(pages)

        signal_sender = Signal(signals.AFTER_ALL_ARTICLES_READ)
        signal_sender.send(context=CONTEXT, articles=CONTEXT.articles)

        signal_sender = Signal(signals.BEFORE_WRITING)
        signal_sender.send(context=CONTEXT, Writer=Writer)

        logger.info("Writing %d articles", len(CONTEXT.articles))
        time_write_start = time.time()
        for index, article in enumerate(CONTEXT.articles):
            logger.debug("Writing file %d of %d", index + 1, len(CONTEXT.articles))
            article.write_file(context=CONTEXT)
        logger.info("Wrote %d articles in %.3f seconds", len(CONTEXT.articles), (time.time() - time_write_start))

        logger.info("Writing %d pages", len(pages))
        time_write_start = time.time()
        for index, page in enumerate(pages):
            logger.debug("Writing file %d of %d", index + 1, len(pages))
            page.write_file(context=CONTEXT)
        logger.info("Wrote %d pages in %.3f seconds", len(pages), (time.time() - time_write_start))

        # Index
        logger.info("Writing articles index")
        index = Index(CONTEXT)
        index.write_file()

        signal_sender = Signal(signals.AFTER_WRITING)
        signal_sender.send(context=CONTEXT, Writer=Writer)

        # Static files
        logger.info("Compiling static files")
        time_static_start = time.time()
        sass.compile(dirname=(CONTEXT.CSS_FOLDER, CONTEXT.OUTPUT_CSS_FOLDER), output_style='compressed')
        for dirname, dirs, files in os.walk(CONTEXT.CSS_FOLDER):
            for filename in files:
                filepath = os.path.join(dirname, filename)
                basename, extension = os.path.splitext(filename)
                rel_path = os.path.relpath(filepath, CONTEXT.CSS_FOLDER)
                if extension.lower() == ".css":
                    os.makedirs(os.path.dirname(os.path.join(CONTEXT.OUTPUT_CSS_FOLDER, rel_path)), exist_ok=True)
                    copyfile(filepath, os.path.join(CONTEXT.OUTPUT_CSS_FOLDER, rel_path))
        for dirname, dirs, files in os.walk(CONTEXT.JS_FOLDER):
            for filename in files:
                filepath = os.path.join(dirname, filename)
                basename, extension = os.path.splitext(filename)
                rel_path = os.path.relpath(filepath, CONTEXT.JS_FOLDER)
                if extension.lower() == ".js":
                    with codecs.open(filepath, encoding='utf-8', errors='ignore') as js_file:
                        minified = js_file.read()
                    output_filepath = os.path.join(CONTEXT.OUTPUT_JS_FOLDER, rel_path)
                    os.makedirs(os.path.dirname(output_filepath), exist_ok=True)
                    with codecs.open(output_filepath, 'w+', encoding='utf-8') as js_min_file:
                        js_min_file.write(minified)
        logger.info("Processed static files in %.3f seconds", time.time() - time_static_start)

        self.subsites = subsites


import os
import datetime
import codecs
import re
from urllib.parse import urljoin

from olm.constants import ArticleStatus
from olm.signals import Signal, signals
from olm.reader import Reader
from olm.writer import Writer

class Article:
    """Object representing an article"""
    def __repr__(self):
        return self.output_filepath

    def __init__(self, context, filepath=None, metadata=None, content=None, basename=None):
        if filepath is not None:
            # Get filenames, paths etc
            dirname = os.path.dirname(filepath)
            basepath, filename = os.path.split(filepath)
            basename, extension = os.path.splitext(filename)
            relpath = os.path.relpath(os.path.join(dirname, basename) + '.html', context.SOURCE_FOLDER)
            
            # Parse the file for content and metadata
            with codecs.open(filepath, 'r', encoding='utf8') as md_file:
                reader = Reader(md_file.read())
                metadata, raw_content = reader.parse_meta()

        elif metadata is not None and content is not None and basename is not None:
            raw_content = content
        else:
            raise Exception('Article object not supplied with either filepath or content and metadata.') 
        
        self.content = context.MD(raw_content)
        self.metadata = metadata

        # Set article variables from metadata
        self.date       = datetime.datetime.strptime(self.metadata['date'].strip(), '%Y-%m-%d') if 'date' in self.metadata else datetime.datetime.now()
        self.type       = self.metadata['type'].strip().lower() if 'type' in self.metadata else None
        self.title      = self.metadata['title'] if 'title' in self.metadata else basename
        self.summary    = context.MD(self.metadata['summary']) if 'summary' in self.metadata else None
        self.location   = self.metadata['location'].strip().lower() if 'location' in self.metadata else None
        status          = self.metadata['status'].strip().lower() if 'status' in self.metadata else None
        if status == 'unlisted' or self.type == 'unlisted':
            self.status = ArticleStatus.UNLISTED
        elif status == 'draft':
            self.status = ArticleStatus.DRAFT
        else:
            self.status = ArticleStatus.ACTIVE
        if 'authors' in self.metadata:
            self.authors = [a.strip() for a in self.metadata['authors'].split(',')]
        elif 'author' in self.metadata:
            self.authors = [a.strip() for a in self.metadata['author'].split(',')]
        else:
            self.authors = []
        for author in self.authors:
            if author in context['authors']:
                context['authors'][author].append(self)
            else:
                context['authors'][author] = [self]
        # Work out other variables
        self.template        = 'article.html'
        self.source_filepath = filepath
        if self.date and self.location:
            output_filename = '{}-{}.html'.format(self.location.lower(), self.date.strftime('%Y-%m-%d'))
        else:
            output_filename = '{}.html'.format(basename)

        self.output_filepath = os.path.join(context.OUTPUT_FOLDER, 'articles', output_filename)
        self.url             = 'articles/{}'.format(output_filename)
        self.context = context
        signal_sender = Signal(signals.AFTER_ARTICLE_READ)
        signal_sender.send(context=context, article=self)

    def write_file(self, context=None):
        self.context = context if context is not None else self.context
        """Write the article to a file"""
        signal_sender = Signal(signals.BEFORE_ARTICLE_WRITE)
        signal_sender.send(context=self.context, article=self)
        writer = Writer(
            self.context, 
            self.output_filepath, 
            self.template,
            article=self)
        writer.write_file()


import os
from os.path import dirname, splitext, relpath, join
import datetime
import codecs
import re
from olm.signals import signals, Signal
from olm.reader import Reader
from olm.writer import Writer

class Page:
    """Object representing an article"""

    def __init__(self, context, filepath=None):
        dirname = os.path.dirname(filepath)
        basepath, filename = os.path.split(filepath)
        basename, extension = os.path.splitext(filename)
        relpath = os.path.relpath(os.path.join(dirname, basename) + '.html', context.SOURCE_FOLDER)
        
        # Parse the file for content and metadata
        with codecs.open(filepath, 'r', encoding='utf8') as md_file:
            reader = Reader(md_file.read())
            metadata, raw_content = reader.parse_meta()

        self.content = context.MD(raw_content)
        self.metadata = metadata

        self.source_filepath = filepath
        self.template        = 'page.html'
        self.title           = self.metadata['title'] if 'title' in self.metadata else basename
        self.url             = relpath
        self.output_filepath = join(context.OUTPUT_FOLDER, relpath)
        self.context = context

        signal_sender = Signal(signals.AFTER_PAGE_READ)
        signal_sender.send(context=context, page=self)


    def write_file(self, context=None):
        self.context = context if context is not None else self.context
        writer = Writer(
            self.context, 
            self.output_filepath, 
            self.template,
            page=self)
        writer.write_file()


import sys
import os
import time
import argparse

from olm.context import load_context, load_default_context
from olm.plugins import Plugins
from olm.signals import Signal, signals
from olm.logger import get_logger
from olm.website import Site

logger = get_logger('olm')

def main():
    parser = argparse.ArgumentParser(description='Olm static site generator',)
    parser.add_argument('src', action="store", help='Path to site folder')
    parser.add_argument('-s', action="store", default=None, help='Path to settings.py file')
    args = parser.parse_args()

    if len(sys.argv) < 2:
        logger.warn("Olm requires the path to the site folder")
        logger.warn("\tolm ./path/to/my/site/")
        return

    time_all = time.time()
    """Main olm function"""
    logger.notice("Beginning static site generation")

    CONTEXT = load_default_context(args.src)

    if args.s is not None:
        settings_file_path = os.path.abspath(args.s)
    else:
        settings_file_path = os.path.abspath(os.path.join(args.src, 'settings.py'))
    if os.path.isfile(settings_file_path):
        CONTEXT = load_context(CONTEXT, settings_file_path=settings_file_path)
    else:
        logger.error('No valid settings.py file found')
        sys.exit()

    plugins = Plugins(CONTEXT)

    signal_sender = Signal(signals.INITIALISED)
    signal_sender.send(context=CONTEXT)

    site = Site(CONTEXT)
    site.build_site()

    subsites = site.subsites

    base_folder = CONTEXT.BASE_FOLDER
    source_folder = CONTEXT.SOURCE_FOLDER
    for subsite in subsites:
        plugins.unload_plugins()
        subsite_name = subsite[1:]
        logger.info("Found subsite '%s'", subsite_name)
        if subsite_name in CONTEXT.SUBSITES:
            subsite_context = load_context(CONTEXT, settings=CONTEXT.SUBSITES[subsite_name])
        else:
            subsite_context = CONTEXT
        plugins.load_plugins(subsite_context)
        subsite_context.OUTPUT_FOLDER = os.path.abspath(os.path.join(base_folder, 'dist', subsite_name))
        subsite_context.BASE_FOLDER = os.path.join(source_folder, subsite)
        subsite_context.SOURCE_FOLDER = os.path.join(source_folder, subsite)
        site = Site(subsite_context)
        site.build_site()

    logger.success("Completed everything in %.3f seconds", (time.time() - time_all))

if __name__== "__main__":
  main()


import re

INDENTATION = re.compile(r'^(\s{4,}|\t{2,})(\S.*)')
META = re.compile(r'^(\w+):\s*(.*)')

class Reader:
    def __init__(self, text):
        self.text = text
        self.metadata = {}
        self.content = ""

    def parse_meta(self):
        """Parse the given text into metadata and strip it for a Markdown parser.
        :param text: text to be parsed
        """
        text = self.text.split('\n')
        meta = {}
        m = META.match(text[0])

        while m:
            key = m.group(1)
            value = m.group(2)
            meta[key] = value
            text = text[1:]
            if len(text) >= 1:
                i = INDENTATION.match(text[0])
                while i:
                    if not isinstance(meta[key], list):
                        meta[key] = [meta[key]]
                    meta[key].append(i.group(2))
                    text = text[1:]
                    i = INDENTATION.match(text[0])
                m = META.match(text[0])
            else:
                m = False

        for key in meta:
            self.metadata[key.lower()] = meta[key].strip() if isinstance(meta[key], str) else meta[key]
        self.content = '\n'.join(text)

        return self.metadata, self.content 